@@ -40,7 +40,6 @@ static void on_success(pa_operation *o, void *userdata)
 {
 	struct success_ack *d = userdata;
 	pa_context *c = o->context;
-	pw_log_debug("error:%d", d->error);
 	if (d->error != 0)
 		pa_context_set_error(c, d->error);
 	if (d->cb)
@@ -101,9 +100,8 @@ static int sink_callback(pa_context *c, struct global *g, struct sink_data *d)
 	struct pw_node_info *info = g->info;
 	const char *str;
 	uint32_t n, j;
+	char monitor_name[1024];
 	pa_sink_info i;
-	pa_format_info ii[1];
-	pa_format_info *ip[1];
 
 	spa_zero(i);
 	if (info->props && (str = spa_dict_lookup(info->props, PW_KEY_NODE_NAME)))
@@ -125,14 +123,16 @@ static int sink_callback(pa_context *c, struct global *g, struct sink_data *d)
 	if (i.sample_spec.channels == g->node_info.channel_map.channels)
 		i.channel_map = g->node_info.channel_map;
 	else
-		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
-	i.owner_module = 0;
+		pa_channel_map_init_extend(&i.channel_map,
+				i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
+	i.owner_module = g->id;
 	i.volume.channels = i.sample_spec.channels;
 	for (n = 0; n < i.volume.channels; n++)
 		i.volume.values[n] = g->node_info.volume * g->node_info.channel_volumes[n] * PA_VOLUME_NORM;
 	i.mute = g->node_info.mute;
 	i.monitor_source = g->node_info.monitor;
-	i.monitor_source_name = pa_context_find_global_name(c, i.monitor_source);
+	snprintf(monitor_name, sizeof(monitor_name)-1, "%s.monitor", i.name);
+	i.monitor_source_name = monitor_name;
 	i.latency = 0;
 	i.driver = "PipeWire";
 	i.flags = PA_SINK_LATENCY | PA_SINK_DYNAMIC_LATENCY |
@@ -182,14 +182,10 @@ static int sink_callback(pa_context *c, struct global *g, struct sink_data *d)
 		else
 			i.ports[j] = NULL;
 	}
-	i.n_formats = 1;
-	ii[0].encoding = PA_ENCODING_PCM;
-	ii[0].plist = pa_proplist_new();
-	ip[0] = ii;
-	i.formats = ip;
+	i.n_formats = pw_array_get_len(&g->node_info.formats, pa_format_info *);
+	i.formats = g->node_info.formats.data;
 	d->cb(c, &i, 0, d->userdata);
 	pa_proplist_free(i.proplist);
-	pa_proplist_free(ii[0].plist);
 	return 0;
 }
 
@@ -200,6 +196,8 @@ static void sink_info(pa_operation *o, void *userdata)
 	pa_context *c = o->context;
 	int error = 0;
 
+	pw_log_debug("%p name:%s idx:%u", c, d->name, d->idx);
+
 	if (d->name) {
 		g = pa_context_find_global_by_name(c, PA_SUBSCRIPTION_MASK_SINK, d->name);
 		pa_xfree(d->name);
@@ -209,12 +207,10 @@ static void sink_info(pa_operation *o, void *userdata)
 			g = NULL;
 	}
 
-	pw_log_debug("%p", c);
-
 	if (g) {
 		error = sink_callback(c, g, d);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
 		pa_context_set_error(c, error);
@@ -235,7 +231,7 @@ pa_operation* pa_context_get_sink_info_by_name(pa_context *c, const char *name,
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
-	pw_log_debug("%p", c);
+	pw_log_debug("%p: name %s", c, name);
 
 	o = pa_operation_new(c, NULL, sink_info, sizeof(struct sink_data));
 	d = o->userdata;
@@ -260,6 +256,8 @@ pa_operation* pa_context_get_sink_info_by_index(pa_context *c, uint32_t idx, pa_
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("%p: index %u", c, idx);
+
 	o = pa_operation_new(c, NULL, sink_info, sizeof(struct sink_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -297,6 +295,7 @@ pa_operation* pa_context_get_sink_info_list(pa_context *c, pa_sink_info_cb_t cb,
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 
+	pw_log_debug("%p", c);
 	o = pa_operation_new(c, NULL, sink_info_list, sizeof(struct sink_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -491,7 +490,7 @@ static void do_node_volume_mute(pa_operation *o, void *userdata)
 				d->have_volume ? g->node_info.mute : d->mute,
 				d->mask);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error != 0)
 		pa_context_set_error(c, error);
@@ -509,12 +508,12 @@ pa_operation* pa_context_set_sink_volume_by_index(pa_context *c, uint32_t idx, c
 	pa_assert(c);
 	pa_assert(c->refcount >= 1);
 
-	pw_log_debug("context %p: index %d", c, idx);
-
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, pa_cvolume_valid(volume), PA_ERR_INVALID);
 
+	pw_log_debug("context %p: index %d", c, idx);
+
 	o = pa_operation_new(c, NULL, do_node_volume_mute, sizeof(struct volume_data));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SINK;
@@ -613,6 +612,7 @@ pa_operation* pa_context_suspend_sink_by_name(pa_context *c, const char *sink_na
 	pa_operation *o;
 	struct success_ack *d;
 
+	pw_log_debug("context %p: name:%s suspend:%d", c, sink_name, suspend);
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -629,6 +629,7 @@ pa_operation* pa_context_suspend_sink_by_index(pa_context *c, uint32_t idx, int
 	pa_operation *o;
 	struct success_ack *d;
 
+	pw_log_debug("context %p: index:%u suspend:%d", c, idx, suspend);
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -668,7 +669,7 @@ static int set_device_route(pa_context *c, struct global *g, const char *port, e
 			pw_log_warn("device %d: can't parse route", g->id);
 			continue;
 		}
-		pw_log_info("port id:%u name:\"%s\" dir:%d", test_id, name, test_direction);
+		pw_log_debug("port id:%u name:\"%s\" dir:%d", test_id, name, test_direction);
 		if (test_direction != direction)
 			continue;
 		if (strcmp(name, port) == 0) {
@@ -676,7 +677,7 @@ static int set_device_route(pa_context *c, struct global *g, const char *port, e
 			break;
 		}
 	}
-	pw_log_info("port %s, id %u", port, id);
+	pw_log_debug("port %s, id %u", port, id);
 	if (id == SPA_ID_INVALID)
 		return PA_ERR_NOENTITY;
 
@@ -724,14 +725,14 @@ static void do_device_route(pa_operation *o, void *userdata)
 	if (g) {
 		error = set_device_route(c, g, d->port, d->direction);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error != 0)
 		pa_context_set_error(c, error);
 	if (d->cb)
 		d->cb(c, error != 0 ? 0 : 1, d->userdata);
-	pa_operation_done(o);
 	pa_xfree(d->port);
+	pa_operation_done(o);
 }
 
 SPA_EXPORT
@@ -743,6 +744,7 @@ pa_operation* pa_context_set_sink_port_by_index(pa_context *c, uint32_t idx, con
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("context %p: index:%u port:%s", c, idx, port);
 	o = pa_operation_new(c, NULL, do_device_route, sizeof(struct device_route));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SINK;
@@ -764,6 +766,7 @@ pa_operation* pa_context_set_sink_port_by_name(pa_context *c, const char*name, c
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
+	pw_log_debug("context %p: name:%s port:%s", c, name, port);
 	o = pa_operation_new(c, NULL, do_device_route, sizeof(struct device_route));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SINK;
@@ -808,8 +811,6 @@ static int source_callback(pa_context *c, struct global *g, struct source_data *
 	const char *str;
 	uint32_t n, j;
 	pa_source_info i;
-	pa_format_info ii[1];
-	pa_format_info *ip[1];
 	enum pa_source_flags flags;
 	bool monitor;
 
@@ -847,8 +848,9 @@ static int source_callback(pa_context *c, struct global *g, struct source_data *
 	if (i.sample_spec.channels == g->node_info.channel_map.channels)
 		i.channel_map = g->node_info.channel_map;
 	else
-		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
-	i.owner_module = 0;
+		pa_channel_map_init_extend(&i.channel_map,
+				i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
+	i.owner_module = g->id;
 	i.volume.channels = i.sample_spec.channels;
 	for (n = 0; n < i.volume.channels; n++)
 		i.volume.values[n] = g->node_info.volume * g->node_info.channel_volumes[n] * PA_VOLUME_NORM;
@@ -907,14 +909,10 @@ static int source_callback(pa_context *c, struct global *g, struct source_data *
 		else
 			i.ports[j] = NULL;
 	}
-	i.n_formats = 1;
-	ii[0].encoding = PA_ENCODING_PCM;
-	ii[0].plist = pa_proplist_new();
-	ip[0] = ii;
-	i.formats = ip;
+	i.n_formats = pw_array_get_len(&g->node_info.formats, pa_format_info *);
+	i.formats = g->node_info.formats.data;
 	d->cb(c, &i, 0, d->userdata);
 	pa_proplist_free(i.proplist);
-	pa_proplist_free(ii[0].plist);
 	return 0;
 }
 
@@ -931,7 +929,7 @@ static void source_info(pa_operation *o, void *userdata)
 	} else {
 		if (((g = pa_context_find_global(c, d->idx)) == NULL ||
 		    !(g->mask & PA_SUBSCRIPTION_MASK_SOURCE)) &&
-		    (((g = pa_context_find_global(c, d->idx & PA_IDX_MASK_DSP)) == NULL ||
+		    (((g = pa_context_find_global(c, d->idx & PA_IDX_MASK_MONITOR)) == NULL ||
 		    !(g->mask & PA_SUBSCRIPTION_MASK_SOURCE))))
 			g = NULL;
 	}
@@ -939,7 +937,7 @@ static void source_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = source_callback(c, g, d);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
 		pa_context_set_error(c, error);
@@ -960,6 +958,7 @@ pa_operation* pa_context_get_source_info_by_name(pa_context *c, const char *name
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
+	pw_log_debug("context %p: name:%s", c, name);
 	o = pa_operation_new(c, NULL, source_info, sizeof(struct source_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1021,6 +1020,7 @@ pa_operation* pa_context_get_source_info_list(pa_context *c, pa_source_info_cb_t
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 
+	pw_log_debug("context %p", c);
 	o = pa_operation_new(c, NULL, source_info_list, sizeof(struct source_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1039,12 +1039,12 @@ pa_operation* pa_context_set_source_volume_by_index(pa_context *c, uint32_t idx,
 	pa_assert(c);
 	pa_assert(c->refcount >= 1);
 
-	pw_log_debug("context %p: index %d", c, idx);
-
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, pa_cvolume_valid(volume), PA_ERR_INVALID);
 
+	pw_log_debug("context %p: index %d", c, idx);
+
 	o = pa_operation_new(c, NULL, do_node_volume_mute, sizeof(struct volume_data));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SOURCE;
@@ -1144,6 +1144,8 @@ pa_operation* pa_context_suspend_source_by_name(pa_context *c, const char *sourc
 	pa_operation *o;
 	struct success_ack *d;
 
+	pw_log_debug("context %p: name:%s", c, source_name);
+
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -1160,6 +1162,7 @@ pa_operation* pa_context_suspend_source_by_index(pa_context *c, uint32_t idx, in
 	pa_operation *o;
 	struct success_ack *d;
 
+	pw_log_debug("context %p: index:%u", c, idx);
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -1179,7 +1182,7 @@ pa_operation* pa_context_set_source_port_by_index(pa_context *c, uint32_t idx, c
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
-	pw_log_debug("context %p: idx %d", c, idx);
+	pw_log_debug("context %p: idx %d port:%s", c, idx, port);
 
 	o = pa_operation_new(c, NULL, do_device_route, sizeof(struct device_route));
 	d = o->userdata;
@@ -1202,7 +1205,7 @@ pa_operation* pa_context_set_source_port_by_name(pa_context *c, const char*name,
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
-	pw_log_debug("context %p: name %s", c, name);
+	pw_log_debug("context %p: name %s port:%s", c, name, port);
 
 	o = pa_operation_new(c, NULL, do_device_route, sizeof(struct device_route));
 	d = o->userdata;
@@ -1253,12 +1256,15 @@ static void server_callback(struct server_data *d, pa_context *c)
 	const struct pw_core_info *info = c->core_info;
 	const char *str;
 	pa_server_info i;
+	char name[1024];
+
+	snprintf(name, sizeof(name)-1, "pulseaudio (on PipeWire %s)", info->version);
 
 	spa_zero(i);
 	i.user_name = info->user_name;
 	i.host_name = info->host_name;
-	i.server_version = info->version;
-	i.server_name = info->name;
+	i.server_version = pa_get_headers_version();
+	i.server_name = name;
 	i.sample_spec.format = PA_SAMPLE_FLOAT32NE;
 	if (info->props && (str = spa_dict_lookup(info->props, "default.clock.rate")) != NULL)
 		i.sample_spec.rate = atoi(str);
@@ -1268,7 +1274,7 @@ static void server_callback(struct server_data *d, pa_context *c)
 	i.default_sink_name = get_default_name(c, PA_SUBSCRIPTION_MASK_SINK);
 	i.default_source_name = get_default_name(c, PA_SUBSCRIPTION_MASK_SOURCE);
 	i.cookie = info->cookie;
-        pa_channel_map_init_extend(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+	pa_channel_map_init_extend(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
 	d->cb(c, &i, d->userdata);
 }
 
@@ -1289,6 +1295,7 @@ pa_operation* pa_context_get_server_info(pa_context *c, pa_server_info_cb_t cb,
 	pa_assert(c->refcount >= 1);
 	pa_assert(cb);
 
+	pw_log_debug("context %p", c);
 	o = pa_operation_new(c, NULL, server_info, sizeof(struct server_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1324,7 +1331,7 @@ static void module_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = module_callback(c, d, g);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
 		pa_context_set_error(c, error);
@@ -1344,6 +1351,7 @@ pa_operation* pa_context_get_module_info(pa_context *c, uint32_t idx, pa_module_
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("context %p index:%u", c, idx);
 	o = pa_operation_new(c, NULL, module_info, sizeof(struct module_data));
 	d = o->userdata;
 	d->idx = idx;
@@ -1381,6 +1389,7 @@ pa_operation* pa_context_get_module_info_list(pa_context *c, pa_module_info_cb_t
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 
+	pw_log_debug("context %p", c);
 	o = pa_operation_new(c, NULL, module_info_list, sizeof(struct module_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1390,37 +1399,185 @@ pa_operation* pa_context_get_module_info_list(pa_context *c, pa_module_info_cb_t
 	return o;
 }
 
-struct load_module_ack {
+struct load_module {
 	pa_context_index_cb_t cb;
 	int error;
 	void *userdata;
 	uint32_t idx;
+	struct pw_properties *props;
+	struct pw_proxy *proxy;
+	struct spa_hook listener;
 };
 
+static struct module_info *find_module(pa_context *c, uint32_t idx)
+{
+	struct module_info *m;
+	spa_list_for_each(m, &c->modules, link) {
+		if (m->id == idx)
+			return m;
+	}
+	return NULL;
+}
+
 static void on_load_module(pa_operation *o, void *userdata)
 {
-	struct load_module_ack *d = userdata;
+	struct load_module *d = userdata;
 	pa_context *c = o->context;
-	pw_log_debug("error:%d", d->error);
 	if (d->error != 0)
 		pa_context_set_error(c, d->error);
 	if (d->cb)
 		d->cb(c, d->idx, d->userdata);
+	if (d->props)
+		pw_properties_free(d->props);
+	if (d->proxy)
+		spa_hook_remove(&d->listener);
 	pa_operation_done(o);
 }
 
+static void module_proxy_removed(void *data)
+{
+	struct module_info *m = data;
+	pw_proxy_destroy(m->proxy);
+}
+
+static void module_proxy_destroy(void *data)
+{
+	struct module_info *m = data;
+	spa_hook_remove(&m->listener);
+	spa_list_remove(&m->link);
+	free(m);
+}
+
+static void module_proxy_bound(void *data, uint32_t global_id)
+{
+	struct module_info *m;
+	pa_operation *o = data;
+	pa_context *c = o->context;
+	struct load_module *d = o->userdata;
+	static const struct pw_proxy_events proxy_events = {
+		.removed = module_proxy_removed,
+		.destroy = module_proxy_destroy,
+	};
+	d->idx = global_id;
+
+	m = calloc(1, sizeof(struct module_info));
+	m->id = global_id;
+	m->proxy = d->proxy;
+	pw_proxy_add_listener(m->proxy, &m->listener, &proxy_events, m);
+	spa_list_append(&c->modules, &m->link);
+	on_load_module(o, d);
+}
+
+static void module_proxy_error(void *data, int seq, int res, const char *message)
+{
+	pa_operation *o = data;
+	struct load_module *d = o->userdata;
+	d->error = res;
+	d->idx = PA_INVALID_INDEX;
+	pw_proxy_destroy(d->proxy);
+	on_load_module(o, d);
+}
+
+static void on_null_sink_module(pa_operation *o, void *userdata)
+{
+	struct load_module *d = userdata;
+	pa_context *c = o->context;
+	static const struct pw_proxy_events proxy_events = {
+		.bound = module_proxy_bound,
+		.error = module_proxy_error,
+	};
+
+	if (d->proxy != NULL)
+		return;
+
+	d->proxy = pw_core_create_object(c->core,
+                                "adapter",
+                                PW_TYPE_INTERFACE_Node,
+                                PW_VERSION_NODE,
+                                d->props ? &d->props->dict : NULL, 0);
+
+	pw_proxy_add_listener(d->proxy, &d->listener, &proxy_events, o);
+}
+
+static void add_props(struct pw_properties *props, const char *str)
+{
+	char *s = strdup(str), *p = s, *e, f;
+	const char *k, *v;
+
+	while (*p) {
+		e = strchr(p, '=');
+		if (e == NULL)
+			break;
+		*e = '\0';
+		k = p;
+		p = e+1;
+
+		if (*p == '\"') {
+			p++;
+			f = '\"';
+		} else {
+			f = ' ';
+		}
+		e = strchr(p, f);
+		if (e == NULL)
+			break;
+		*e = '\0';
+		v = p;
+		p = e + 1;
+		pw_properties_set(props, k, v);
+	}
+	free(s);
+}
+
 SPA_EXPORT
 pa_operation* pa_context_load_module(pa_context *c, const char*name, const char *argument, pa_context_index_cb_t cb, void *userdata)
 {
 	pa_operation *o;
-	struct load_module_ack *d;
+	struct load_module *d;
+	int error = PA_ERR_NOTIMPLEMENTED;;
+	struct pw_properties *props = NULL;
+	pa_operation_cb_t op_cb = on_load_module;
+	const char *str;
 
-	o = pa_operation_new(c, NULL, on_load_module, sizeof(struct success_ack));
+	pa_assert(c);
+	pa_assert(c->refcount >= 1);
+	pa_assert(name != NULL);
+
+	pw_log_debug("context %p: name:%s arg:%s", c, name, argument);
+
+	if (strcmp(name, "module-null-sink") == 0) {
+		props = pw_properties_new_string(argument);
+		if (props == NULL) {
+			error = PA_ERR_INVALID;
+			goto done;
+		}
+		if ((str = pw_properties_get(props, "sink_name")) != NULL) {
+			pw_properties_set(props, "node.name", str);
+			pw_properties_set(props, "sink_name", NULL);
+		} else {
+			pw_properties_set(props, "node.name", "null");
+		}
+		if ((str = pw_properties_get(props, "sink_properties")) != NULL) {
+			add_props(props, str);
+			pw_properties_set(props, "sink_properties", NULL);
+		}
+		if ((str = pw_properties_get(props, "device.description")) != NULL) {
+			pw_properties_set(props, "node.description", str);
+			pw_properties_set(props, "device.description", NULL);
+		}
+		pw_properties_set(props, "factory.name", "support.null-audio-sink");
+
+		error = 0;
+		op_cb = on_null_sink_module;
+	}
+done:
+	o = pa_operation_new(c, NULL, op_cb, sizeof(struct load_module));
 	d = o->userdata;
 	d->cb = cb;
-	d->error = PA_ERR_NOTIMPLEMENTED;
+	d->error = error;
 	d->userdata = userdata;
 	d->idx = PA_INVALID_INDEX;
+	d->props = props;
 	pa_operation_sync(o);
 
 	return o;
@@ -1431,12 +1588,22 @@ pa_operation* pa_context_unload_module(pa_context *c, uint32_t idx, pa_context_s
 {
 	pa_operation *o;
 	struct success_ack *d;
+	struct module_info *m;
+	int error;
 
+	pw_log_debug("context %p: %u", c, idx);
+	if ((m = find_module(c, idx)) != NULL) {
+		pw_proxy_destroy(m->proxy);
+		error = 0;
+	} else {
+		error = PA_ERR_NOENTITY;
+	}
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
-	d->error = PA_ERR_NOTIMPLEMENTED;
 	d->userdata = userdata;
+	d->error = error;
+	d->idx = idx;
 	pa_operation_sync(o);
 
 	return o;
@@ -1468,7 +1635,7 @@ static void client_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = client_callback(c, d, g);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
 		pa_context_set_error(c, error);
@@ -1488,6 +1655,7 @@ pa_operation* pa_context_get_client_info(pa_context *c, uint32_t idx, pa_client_
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("context %p: index:%u", c, idx);
 	o = pa_operation_new(c, NULL, client_info, sizeof(struct client_data));
 	d = o->userdata;
 	d->idx = idx;
@@ -1525,6 +1693,7 @@ pa_operation* pa_context_get_client_info_list(pa_context *c, pa_client_info_cb_t
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 
+	pw_log_debug("context %p", c);
 	o = pa_operation_new(c, NULL, client_info_list, sizeof(struct client_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1554,7 +1723,7 @@ static void do_kill_client(pa_operation *o, void *userdata)
 	if (g) {
 		pw_registry_destroy(c->registry, g->id);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error != 0)
 		pa_context_set_error(c, error);
@@ -1571,6 +1740,7 @@ pa_operation* pa_context_kill_client(pa_context *c, uint32_t idx, pa_context_suc
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("context %p: index:%u", c, idx);
 	o = pa_operation_new(c, NULL, do_kill_client, sizeof(struct kill_client));
 	d = o->userdata;
 	d->idx = idx;
@@ -1614,7 +1784,7 @@ static void card_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = card_callback(c, d, g);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error != 0)
 		pa_context_set_error(c, error);
@@ -1636,7 +1806,7 @@ pa_operation* pa_context_get_card_info_by_index(pa_context *c, uint32_t idx, pa_
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
-	pw_log_debug("context %p: %u", c, idx);
+	pw_log_debug("context %p: index:%u", c, idx);
 	o = pa_operation_new(c, NULL, card_info, sizeof(struct card_data));
 	d = o->userdata;
 	d->idx = idx;
@@ -1659,7 +1829,7 @@ pa_operation* pa_context_get_card_info_by_name(pa_context *c, const char *name,
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
-	pw_log_debug("context %p: %s", c, name);
+	pw_log_debug("context %p: name:%s", c, name);
 	o = pa_operation_new(c, NULL, card_info, sizeof(struct card_data));
 	d = o->userdata;
 	d->name = pa_xstrdup(name);
@@ -1725,7 +1895,7 @@ static void card_profile(pa_operation *o, void *userdata)
 		g = NULL;
 
 	if (g == NULL) {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 		goto done;
 	}
 
@@ -1746,7 +1916,7 @@ static void card_profile(pa_operation *o, void *userdata)
 		}
 	}
 	if (id == SPA_ID_INVALID) {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 		goto done;
 	}
 
@@ -1765,8 +1935,8 @@ done:
 		pa_context_set_error(c, error);
 	if (d->success_cb)
 		d->success_cb(c, error ? 0 : 1, d->userdata);
-	pa_operation_done(o);
 	free(d->profile);
+	pa_operation_done(o);
 }
 
 SPA_EXPORT
@@ -1781,7 +1951,7 @@ pa_operation* pa_context_set_card_profile_by_index(pa_context *c, uint32_t idx,
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
-	pw_log_debug("Card set profile %s", profile);
+	pw_log_debug("%p: index:%u profile:%s", c, idx, profile);
 	o = pa_operation_new(c, NULL, card_profile, sizeof(struct card_data));
 	d = o->userdata;
 	d->idx = idx;
@@ -1805,7 +1975,7 @@ pa_operation* pa_context_set_card_profile_by_name(pa_context *c, const char*name
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(c, !name || *name, PA_ERR_INVALID);
 
-	pw_log_debug("Card set profile %s", profile);
+	pw_log_debug("%p: name:%s profile:%s", c, name, profile);
 	o = pa_operation_new(c, NULL, card_profile, sizeof(struct card_data));
 	d = o->userdata;
 	d->name = pa_xstrdup(name);
@@ -1823,6 +1993,7 @@ pa_operation* pa_context_set_port_latency_offset(pa_context *c, const char *card
 	pa_operation *o;
 	struct success_ack *d;
 
+	pw_log_debug("%p: card_name:%s port_name:%s offset:%"PRIi64, c, card_name, port_name, offset);
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -1833,16 +2004,6 @@ pa_operation* pa_context_set_port_latency_offset(pa_context *c, const char *card
 	return o;
 }
 
-static pa_stream *find_stream(pa_context *c, uint32_t idx)
-{
-	pa_stream *s;
-	spa_list_for_each(s, &c->streams, link) {
-		if (pw_stream_get_node_id(s->stream) == idx)
-			return s;
-	}
-	return NULL;
-}
-
 struct sink_input_data {
 	pa_sink_input_info_cb_t cb;
 	uint32_t idx;
@@ -1862,7 +2023,7 @@ static int sink_input_callback(pa_context *c, struct sink_input_data *d, struct
 	if (info == NULL)
 		return PA_ERR_INVALID;
 
-	s = find_stream(c, g->id);
+	s = pa_context_find_stream(c, g->id);
 
 	if (info->props) {
 		if ((name = spa_dict_lookup(info->props, PW_KEY_MEDIA_NAME)) == NULL &&
@@ -1878,23 +2039,20 @@ static int sink_input_callback(pa_context *c, struct sink_input_data *d, struct
 	spa_zero(i);
 	i.index = g->id;
 	i.name = name;
-	i.owner_module = PA_INVALID_INDEX;
+	i.owner_module = g->id;
 	i.client = g->node_info.client_id;
-	if (s) {
+	if (s)
 		i.sink = s->device_index;
-	}
-	else {
-		struct global *l;
-		l = pa_context_find_linked(c, g->id);
-		i.sink = l ? l->id : PA_INVALID_INDEX;
-	}
+	else
+		i.sink = g->node_info.device_index;
+
 	if (s && s->sample_spec.channels > 0) {
 		i.sample_spec = s->sample_spec;
 		if (s->channel_map.channels == s->sample_spec.channels)
 			i.channel_map = s->channel_map;
 		else
-			pa_channel_map_init_auto(&i.channel_map,
-					i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+			pa_channel_map_init_extend(&i.channel_map,
+					i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
 		i.format = s->format;
 	}
 	else {
@@ -1906,7 +2064,8 @@ static int sink_input_callback(pa_context *c, struct sink_input_data *d, struct
 		if (i.sample_spec.channels == g->node_info.channel_map.channels)
 			i.channel_map = g->node_info.channel_map;
 		else
-			pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+			pa_channel_map_init_extend(&i.channel_map,
+					i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
 		ii[0].encoding = PA_ENCODING_PCM;
 		ii[0].plist = pa_proplist_new();
 		i.format = ii;
@@ -1950,10 +2109,10 @@ static void sink_input_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = sink_input_callback(c, d, g);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
-		pa_context_set_error(c, PA_ERR_INVALID);
+		pa_context_set_error(c, error);
 	d->cb(c, NULL, error ? -1 : 1, d->userdata);
 	pa_operation_done(o);
 }
@@ -2069,8 +2228,8 @@ done:
 		pa_context_set_error(c, error);
 	if (d->cb)
 		d->cb(c, error != 0 ? 0 : 1, d->userdata);
-	pa_operation_done(o);
 	pa_xfree(d->target_name);
+	pa_operation_done(o);
 }
 
 SPA_EXPORT
@@ -2079,6 +2238,7 @@ pa_operation* pa_context_move_sink_input_by_name(pa_context *c, uint32_t idx, co
 	pa_operation *o;
 	struct target_node *d;
 
+	pw_log_debug("%p: index:%u name:%s", c, idx, sink_name);
 	o = pa_operation_new(c, NULL, do_target_node, sizeof(struct target_node));
 	d = o->userdata;
 	d->idx = idx;
@@ -2099,6 +2259,7 @@ pa_operation* pa_context_move_sink_input_by_index(pa_context *c, uint32_t idx, u
 	pa_operation *o;
 	struct target_node *d;
 
+	pw_log_debug("%p: index:%u sink_index:%u", c, idx, sink_idx);
 	o = pa_operation_new(c, NULL, do_target_node, sizeof(struct target_node));
 	d = o->userdata;
 	d->idx = idx;
@@ -2131,7 +2292,7 @@ static void do_stream_volume_mute(pa_operation *o, void *userdata)
 	int error = 0;
 	pa_stream *s;
 
-	if ((s = find_stream(c, d->idx)) == NULL) {
+	if ((s = pa_context_find_stream(c, d->idx)) == NULL) {
 		if ((g = pa_context_find_global(c, d->idx)) == NULL ||
 		    !(g->mask & d->mask))
 			g = NULL;
@@ -2145,7 +2306,7 @@ static void do_stream_volume_mute(pa_operation *o, void *userdata)
 				d->have_volume ? &d->volume : NULL,
 				d->have_volume ? g->node_info.mute : d->mute);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 
 	if (error != 0)
@@ -2209,7 +2370,7 @@ static void do_kill_stream(pa_operation *o, void *userdata)
 	int error = 0;
 	pa_stream *s;
 
-	if ((s = find_stream(c, d->idx)) == NULL) {
+	if ((s = pa_context_find_stream(c, d->idx)) == NULL) {
 		if ((g = pa_context_find_global(c, d->idx)) == NULL ||
 		    !(g->mask & d->mask))
 			g = NULL;
@@ -2219,7 +2380,7 @@ static void do_kill_stream(pa_operation *o, void *userdata)
 	} else if (g) {
 		pw_registry_destroy(c->registry, g->id);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error != 0)
 		pa_context_set_error(c, error);
@@ -2254,7 +2415,7 @@ struct source_output_data {
 
 static int source_output_callback(struct source_output_data *d, pa_context *c, struct global *g)
 {
-	struct global *l, *cl;
+	struct global *cl;
 	struct pw_node_info *info = g->info;
 	const char *name = NULL;
 	uint32_t n;
@@ -2266,7 +2427,7 @@ static int source_output_callback(struct source_output_data *d, pa_context *c, s
 	if (info == NULL)
 		return PA_ERR_INVALID;
 
-	s = find_stream(c, g->id);
+	s = pa_context_find_stream(c, g->id);
 
 	if (info->props) {
 		if ((name = spa_dict_lookup(info->props, PW_KEY_MEDIA_NAME)) == NULL &&
@@ -2282,22 +2443,19 @@ static int source_output_callback(struct source_output_data *d, pa_context *c, s
 	spa_zero(i);
 	i.index = g->id;
 	i.name = name;
-	i.owner_module = PA_INVALID_INDEX;
+	i.owner_module = g->id;
 	i.client = g->node_info.client_id;
-	if (s) {
+	if (s)
 		i.source = s->device_index;
-	}
-	else {
-		l = pa_context_find_linked(c, g->id);
-		i.source = l ? l->id : PA_INVALID_INDEX;
-	}
+	else
+		i.source = g->node_info.device_index;
 	if (s && s->sample_spec.channels > 0) {
 		i.sample_spec = s->sample_spec;
 		if (s->channel_map.channels == s->sample_spec.channels)
 			i.channel_map = s->channel_map;
 		else
-			pa_channel_map_init_auto(&i.channel_map,
-					i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+			pa_channel_map_init_extend(&i.channel_map,
+					i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
 		i.format = s->format;
 	}
 	else {
@@ -2309,7 +2467,8 @@ static int source_output_callback(struct source_output_data *d, pa_context *c, s
 		if (i.sample_spec.channels == g->node_info.channel_map.channels)
 			i.channel_map = g->node_info.channel_map;
 		else
-			pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+			pa_channel_map_init_extend(&i.channel_map,
+					i.sample_spec.channels, PA_CHANNEL_MAP_DEFAULT);
 		ii[0].encoding = PA_ENCODING_PCM;
 		ii[0].plist = pa_proplist_new();
 		i.format = ii;
@@ -2351,7 +2510,7 @@ static void source_output_info(pa_operation *o, void *userdata)
 	if (g) {
 		error = source_output_callback(d, c, g);
 	} else {
-		error = PA_ERR_INVALID;
+		error = PA_ERR_NOENTITY;
 	}
 	if (error)
 		pa_context_set_error(c, error);
@@ -2371,6 +2530,7 @@ pa_operation* pa_context_get_source_output_info(pa_context *c, uint32_t idx, pa_
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, idx != PA_INVALID_INDEX, PA_ERR_INVALID);
 
+	pw_log_debug("%p: index:%u", c, idx);
 	o = pa_operation_new(c, NULL, source_output_info, sizeof(struct source_output_data));
 	d = o->userdata;
 	d->idx = idx;
@@ -2408,6 +2568,7 @@ pa_operation* pa_context_get_source_output_info_list(pa_context *c, pa_source_ou
 
 	PA_CHECK_VALIDITY_RETURN_NULL(c, c->state == PA_CONTEXT_READY, PA_ERR_BADSTATE);
 
+	pw_log_debug("%p", c);
 	o = pa_operation_new(c, NULL, source_output_info_list, sizeof(struct source_output_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -2423,6 +2584,7 @@ pa_operation* pa_context_move_source_output_by_name(pa_context *c, uint32_t idx,
 	pa_operation *o;
 	struct target_node *d;
 
+	pw_log_debug("%p index:%u name:%s", c, idx, source_name);
 	o = pa_operation_new(c, NULL, do_target_node, sizeof(struct target_node));
 	d = o->userdata;
 	d->idx = idx;
@@ -2443,6 +2605,7 @@ pa_operation* pa_context_move_source_output_by_index(pa_context *c, uint32_t idx
 	pa_operation *o;
 	struct target_node *d;
 
+	pw_log_debug("%p index:%u source_index:%u", c, idx, source_idx);
 	o = pa_operation_new(c, NULL, do_target_node, sizeof(struct target_node));
 	d = o->userdata;
 	d->idx = idx;
@@ -2526,7 +2689,6 @@ static void on_stat_info(pa_operation *o, void *userdata)
 	pa_context *c = o->context;
 	pa_stat_info i;
 	spa_zero(i);
-	pw_log_debug("error:%d", d->error);
 	if (d->error != 0)
 		pa_context_set_error(c, d->error);
 	if (d->cb)
@@ -2540,6 +2702,7 @@ pa_operation* pa_context_stat(pa_context *c, pa_stat_info_cb_t cb, void *userdat
 	pa_operation *o;
 	struct stat_ack *d;
 
+	pw_log_debug("%p", c);
 	o = pa_operation_new(c, NULL, on_stat_info, sizeof(struct stat_ack));
 	d = o->userdata;
 	d->cb = cb;
@@ -2560,7 +2723,6 @@ static void on_sample_info(pa_operation *o, void *userdata)
 {
 	struct sample_info *d = userdata;
 	pa_context *c = o->context;
-	pw_log_debug("error:%d", d->error);
 	if (d->error != 0)
 		pa_context_set_error(c, d->error);
 	if (d->cb)
@@ -2574,6 +2736,7 @@ pa_operation* pa_context_get_sample_info_by_name(pa_context *c, const char *name
 	pa_operation *o;
 	struct sample_info *d;
 
+	pw_log_debug("%p nane:%s", c, name);
 	o = pa_operation_new(c, NULL, on_sample_info, sizeof(struct sample_info));
 	d = o->userdata;
 	d->cb = cb;
@@ -2590,6 +2753,7 @@ pa_operation* pa_context_get_sample_info_by_index(pa_context *c, uint32_t idx, p
 	pa_operation *o;
 	struct sample_info *d;
 
+	pw_log_debug("%p index:%u", c, idx);
 	o = pa_operation_new(c, NULL, on_sample_info, sizeof(struct sample_info));
 	d = o->userdata;
 	d->cb = cb;
@@ -2618,6 +2782,7 @@ pa_operation* pa_context_get_sample_info_list(pa_context *c, pa_sample_info_cb_t
 	pa_operation *o;
 	struct sample_info *d;
 
+	pw_log_debug("%p", c);
 	o = pa_operation_new(c, NULL, on_sample_info_list, sizeof(struct sample_info));
 	d = o->userdata;
 	d->cb = cb;
