@@ -52,15 +52,21 @@
 
 #define MAX_POLL	16
 
-static const char default_device[] = "hw:0";
+#define DEFAULT_DEVICE		"hw:0"
+#define DEFAULT_AUTO_PROFILE	true
+#define DEFAULT_AUTO_PORT	true
 
 struct props {
 	char device[64];
+	bool auto_profile;
+	bool auto_port;
 };
 
 static void reset_props(struct props *props)
 {
-	strncpy(props->device, default_device, 64);
+	strncpy(props->device, DEFAULT_DEVICE, 64);
+	props->auto_profile = DEFAULT_AUTO_PROFILE;
+	props->auto_port = DEFAULT_AUTO_PORT;
 }
 
 struct impl {
@@ -72,6 +78,10 @@ struct impl {
 
 	uint32_t info_all;
 	struct spa_device_info info;
+#define IDX_EnumProfile		0
+#define IDX_Profile		1
+#define IDX_EnumRoute		2
+#define IDX_Route		3
 	struct spa_param_info params[4];
 
 	struct spa_hook_list hooks;
@@ -641,12 +651,13 @@ static void card_profile_changed(void *data, uint32_t old_index, uint32_t new_in
 	setup_sources(this);
 
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[1].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Profile].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
 }
 
 static void card_profile_available(void *data, uint32_t index,
-                enum acp_available old, enum acp_available available)
+		enum acp_available old, enum acp_available available)
 {
 	struct impl *this = data;
 	struct acp_card *card = this->card;
@@ -654,8 +665,14 @@ static void card_profile_available(void *data, uint32_t index,
 	spa_log_info(this->log, "card profile %s available %d", p->name, available);
 
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[0].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_EnumProfile].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Profile].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
+
+	if (this->props.auto_profile) {
+		uint32_t best = acp_card_find_best_profile_index(card, NULL);
+		acp_card_set_profile(card, best);
+	}
 }
 
 static void card_port_changed(void *data, uint32_t old_index, uint32_t new_index)
@@ -669,12 +686,12 @@ static void card_port_changed(void *data, uint32_t old_index, uint32_t new_index
 			op->name, np->name);
 
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[3].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
 }
 
 static void card_port_available(void *data, uint32_t index,
-                enum acp_available old, enum acp_available available)
+		enum acp_available old, enum acp_available available)
 {
 	struct impl *this = data;
 	struct acp_card *card = this->card;
@@ -682,8 +699,24 @@ static void card_port_available(void *data, uint32_t index,
 	spa_log_info(this->log, "card port %s available %d", p->name, available);
 
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[2].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_EnumRoute].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
+
+	if (this->props.auto_port) {
+		uint32_t i;
+
+		for (i = 0; i < p->n_devices; i++) {
+			struct acp_device *d = p->devices[i];
+			uint32_t best;
+
+			if (!(d->flags & ACP_DEVICE_ACTIVE))
+				continue;
+
+			best = acp_device_find_best_port_index(d, NULL);
+			acp_device_set_port(d, best);
+		}
+	}
 }
 
 static void on_volume_changed(void *data, struct acp_device *dev)
@@ -691,7 +724,7 @@ static void on_volume_changed(void *data, struct acp_device *dev)
 	struct impl *this = data;
 	spa_log_info(this->log, "device %s volume changed", dev->name);
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[3].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
 }
 
@@ -700,7 +733,7 @@ static void on_mute_changed(void *data, struct acp_device *dev)
 	struct impl *this = data;
 	spa_log_info(this->log, "device %s mute changed", dev->name);
 	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[3].flags ^= SPA_PARAM_INFO_SERIAL;
+	this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(this, false);
 }
 
@@ -872,10 +905,10 @@ impl_init(const struct spa_handle_factory *factory,
 	this->info_all = SPA_DEVICE_CHANGE_MASK_PROPS |
 		SPA_DEVICE_CHANGE_MASK_PARAMS;
 
-	this->params[0] = SPA_PARAM_INFO(SPA_PARAM_EnumProfile, SPA_PARAM_INFO_READ);
-	this->params[1] = SPA_PARAM_INFO(SPA_PARAM_Profile, SPA_PARAM_INFO_READWRITE);
-	this->params[2] = SPA_PARAM_INFO(SPA_PARAM_EnumRoute, SPA_PARAM_INFO_READ);
-	this->params[3] = SPA_PARAM_INFO(SPA_PARAM_Route, SPA_PARAM_INFO_READWRITE);
+	this->params[IDX_EnumProfile] = SPA_PARAM_INFO(SPA_PARAM_EnumProfile, SPA_PARAM_INFO_READ);
+	this->params[IDX_Profile] = SPA_PARAM_INFO(SPA_PARAM_Profile, SPA_PARAM_INFO_READWRITE);
+	this->params[IDX_EnumRoute] = SPA_PARAM_INFO(SPA_PARAM_EnumRoute, SPA_PARAM_INFO_READ);
+	this->params[IDX_Route] = SPA_PARAM_INFO(SPA_PARAM_Route, SPA_PARAM_INFO_READWRITE);
 	this->info.params = this->params;
 	this->info.n_params = 4;
 
