@@ -51,6 +51,7 @@ static void reset_props(struct props *props)
 	strncpy(props->device, default_device, 64);
 	props->min_latency = default_min_latency;
 	props->max_latency = default_max_latency;
+	props->use_chmap = DEFAULT_USE_CHMAP;
 }
 
 static int impl_node_enum_params(void *object, int seq,
@@ -115,6 +116,13 @@ static int impl_node_enum_params(void *object, int seq,
 				SPA_PROP_INFO_name, SPA_POD_String("The maximum latency"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(p->max_latency, 1, INT32_MAX));
 			break;
+		case 5:
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_PropInfo, id,
+				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_START_CUSTOM),
+				SPA_PROP_INFO_name, SPA_POD_String("Use the driver channelmap"),
+				SPA_PROP_INFO_type, SPA_POD_Bool(p->use_chmap));
+			break;
 		default:
 			return 0;
 		}
@@ -125,11 +133,12 @@ static int impl_node_enum_params(void *object, int seq,
 		case 0:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_Props, id,
-				SPA_PROP_device,      SPA_POD_Stringn(p->device, sizeof(p->device)),
-				SPA_PROP_deviceName,  SPA_POD_Stringn(p->device_name, sizeof(p->device_name)),
-				SPA_PROP_cardName,    SPA_POD_Stringn(p->card_name, sizeof(p->card_name)),
-				SPA_PROP_minLatency,  SPA_POD_Int(p->min_latency),
-				SPA_PROP_maxLatency,  SPA_POD_Int(p->max_latency));
+				SPA_PROP_device,       SPA_POD_Stringn(p->device, sizeof(p->device)),
+				SPA_PROP_deviceName,   SPA_POD_Stringn(p->device_name, sizeof(p->device_name)),
+				SPA_PROP_cardName,     SPA_POD_Stringn(p->card_name, sizeof(p->card_name)),
+				SPA_PROP_minLatency,   SPA_POD_Int(p->min_latency),
+				SPA_PROP_maxLatency,   SPA_POD_Int(p->max_latency),
+				SPA_PROP_START_CUSTOM, SPA_POD_Bool(p->use_chmap));
 			break;
 		default:
 			return 0;
@@ -210,9 +219,10 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 		}
 		spa_pod_parse_object(param,
 			SPA_TYPE_OBJECT_Props, NULL,
-			SPA_PROP_device,     SPA_POD_OPT_Stringn(p->device, sizeof(p->device)),
-			SPA_PROP_minLatency, SPA_POD_OPT_Int(&p->min_latency),
-			SPA_PROP_maxLatency, SPA_POD_OPT_Int(&p->max_latency));
+			SPA_PROP_device,       SPA_POD_OPT_Stringn(p->device, sizeof(p->device)),
+			SPA_PROP_minLatency,   SPA_POD_OPT_Int(&p->min_latency),
+			SPA_PROP_maxLatency,   SPA_POD_OPT_Int(&p->max_latency),
+			SPA_PROP_START_CUSTOM, SPA_POD_OPT_Bool(&p->use_chmap));
 		break;
 	}
 	default:
@@ -772,6 +782,8 @@ impl_init(const struct spa_handle_factory *factory,
 			snprintf(this->props.device, 63, "%s", info->items[i].value);
 		} else if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_CHANNELS)) {
 			this->default_channels = atoi(info->items[i].value);
+		} else if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_RATE)) {
+			this->default_rate = atoi(info->items[i].value);
 		}
 	}
 	return 0;
