@@ -178,9 +178,13 @@ static int setup_convert(struct impl *this)
 			    outformat.info.raw.position[j])
 				continue;
 			this->remap[i] = j;
+			spa_log_debug(this->log, NAME " %p: channel %d -> %d (%s -> %s)", this,
+					i, j,
+					spa_debug_type_find_short_name(spa_type_audio_channel,
+						informat.info.raw.position[i]),
+					spa_debug_type_find_short_name(spa_type_audio_channel,
+						outformat.info.raw.position[i]));
 			outformat.info.raw.position[j] = -1;
-			spa_log_debug(this->log, NAME " %p: channel %d -> %d (%d)", this,
-					i, j, informat.info.raw.position[i]);
 			break;
 		}
 	}
@@ -243,6 +247,8 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	case SPA_NODE_COMMAND_Start:
 		this->started = true;
 		break;
+	case SPA_NODE_COMMAND_Suspend:
+		/* fallthrough */
 	case SPA_NODE_COMMAND_Pause:
 		this->started = false;
 		break;
@@ -317,7 +323,13 @@ impl_node_remove_port(void *object, enum spa_direction direction, uint32_t port_
 
 static int int32_cmp(const void *v1, const void *v2)
 {
-	return *(int32_t*)v1 - *(int32_t*)v2;
+	int32_t a1 = *(int32_t*)v1;
+	int32_t a2 = *(int32_t*)v2;
+	if (a1 == 0 && a2 != 0)
+		return 1;
+	if (a2 == 0 && a1 != 0)
+		return -1;
+	return a1 - a2;
 }
 
 static int port_enum_formats(void *object,
@@ -365,12 +377,12 @@ static int port_enum_formats(void *object,
 								SPA_AUDIO_FORMAT_F32P,
 								SPA_AUDIO_FORMAT_F32,
 								SPA_AUDIO_FORMAT_F32_OE,
-								SPA_AUDIO_FORMAT_S24_32P,
-								SPA_AUDIO_FORMAT_S24_32,
-								SPA_AUDIO_FORMAT_S24_32_OE,
 								SPA_AUDIO_FORMAT_S32P,
 								SPA_AUDIO_FORMAT_S32,
 								SPA_AUDIO_FORMAT_S32_OE,
+								SPA_AUDIO_FORMAT_S24_32P,
+								SPA_AUDIO_FORMAT_S24_32,
+								SPA_AUDIO_FORMAT_S24_32_OE,
 								SPA_AUDIO_FORMAT_S16P,
 								SPA_AUDIO_FORMAT_S16,
 								SPA_AUDIO_FORMAT_S16_OE,
