@@ -347,6 +347,8 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	case SPA_NODE_COMMAND_Start:
 		this->started = true;
 		break;
+	case SPA_NODE_COMMAND_Suspend:
+		/* fallthrough */
 	case SPA_NODE_COMMAND_Pause:
 		this->started = false;
 		break;
@@ -427,18 +429,23 @@ static int port_enum_formats(void *object,
 				SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
 				SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_audio),
 				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
-				SPA_FORMAT_AUDIO_format,   SPA_POD_CHOICE_ENUM_Id(13,
-							SPA_AUDIO_FORMAT_F32,
-							SPA_AUDIO_FORMAT_F32,
+				SPA_FORMAT_AUDIO_format,   SPA_POD_CHOICE_ENUM_Id(18,
 							SPA_AUDIO_FORMAT_F32P,
-							SPA_AUDIO_FORMAT_S32,
+							SPA_AUDIO_FORMAT_F32P,
+							SPA_AUDIO_FORMAT_F32,
+							SPA_AUDIO_FORMAT_F32_OE,
 							SPA_AUDIO_FORMAT_S32P,
-							SPA_AUDIO_FORMAT_S24_32,
+							SPA_AUDIO_FORMAT_S32,
+							SPA_AUDIO_FORMAT_S32_OE,
 							SPA_AUDIO_FORMAT_S24_32P,
-							SPA_AUDIO_FORMAT_S24,
-							SPA_AUDIO_FORMAT_S24P,
-							SPA_AUDIO_FORMAT_S16,
+							SPA_AUDIO_FORMAT_S24_32,
+							SPA_AUDIO_FORMAT_S24_32_OE,
 							SPA_AUDIO_FORMAT_S16P,
+							SPA_AUDIO_FORMAT_S16,
+							SPA_AUDIO_FORMAT_S16_OE,
+							SPA_AUDIO_FORMAT_S24P,
+							SPA_AUDIO_FORMAT_S24,
+							SPA_AUDIO_FORMAT_S24_OE,
 							SPA_AUDIO_FORMAT_U8,
 							SPA_AUDIO_FORMAT_U8P),
 				SPA_FORMAT_AUDIO_rate,     SPA_POD_CHOICE_RANGE_Int(
@@ -970,7 +977,11 @@ static int impl_node_process(void *object)
 	dd = &dbuf->buf->datas[0];
 
 	maxsize = dd->maxsize;
-	n_samples = maxsize / outport->stride;
+
+	if (SPA_LIKELY(this->io_position))
+		n_samples = this->io_position->clock.duration;
+	else
+		n_samples = maxsize / outport->stride;
 
 	src_datas = alloca(sizeof(void*) * this->port_count);
 
