@@ -40,9 +40,6 @@
 
 #include "resample.h"
 
-#include "resample-peaks.h"
-#include "resample-native.h"
-
 #define NAME "resample"
 
 #define DEFAULT_RATE		48000
@@ -121,6 +118,7 @@ struct impl {
 	int mode;
 	unsigned int started:1;
 	unsigned int peaks:1;
+	unsigned int drained:1;
 
 	struct resample resample;
 };
@@ -166,9 +164,9 @@ static int setup_convert(struct impl *this,
 	this->resample.quality = this->props.quality;
 
 	if (this->peaks)
-		err = impl_peaks_init(&this->resample);
+		err = resample_peaks_init(&this->resample);
 	else
-		err = impl_native_init(&this->resample);
+		err = resample_native_init(&this->resample);
 
 	return err;
 }
@@ -240,6 +238,16 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 	return 0;
 }
 
+static void update_rate_match(struct impl *this)
+{
+	if (this->io_rate_match) {
+		this->io_rate_match->delay = resample_delay(&this->resample);
+		if (SPA_LIKELY(this->io_position))
+			this->io_rate_match->size = resample_in_len(&this->resample,
+					this->io_position->clock.duration);
+	}
+}
+
 static int impl_node_send_command(void *object, const struct spa_command *command)
 {
 	struct impl *this = object;
@@ -250,10 +258,13 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	switch (SPA_NODE_COMMAND_ID(command)) {
 	case SPA_NODE_COMMAND_Start:
 		this->started = true;
+		resample_update_rate(&this->resample, 1.0);
+		update_rate_match(this);
 		break;
 	case SPA_NODE_COMMAND_Suspend:
 		/* fallthrough */
 	case SPA_NODE_COMMAND_Pause:
+		resample_reset(&this->resample);
 		this->started = false;
 		break;
 	default:
@@ -726,6 +737,7 @@ static int impl_node_process(void *object)
 	void **dst_datas;
 	bool flush_out = false;
 	bool flush_in = false;
+	bool draining = false;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
@@ -744,15 +756,18 @@ static int impl_node_process(void *object)
 
 	if (SPA_UNLIKELY(outio->status == SPA_STATUS_HAVE_DATA))
 		return SPA_STATUS_HAVE_DATA;
-
-	if (SPA_UNLIKELY(inio->status != SPA_STATUS_HAVE_DATA))
-		return SPA_STATUS_NEED_DATA;
-
 	/* recycle */
 	if (SPA_LIKELY(outio->buffer_id < outport->n_buffers)) {
 		recycle_buffer(this, outio->buffer_id);
 		outio->buffer_id = SPA_ID_INVALID;
 	}
+	if (SPA_UNLIKELY(inio->status != SPA_STATUS_HAVE_DATA)) {
+		if (inio->status != SPA_STATUS_DRAINED || this->drained)
+			return outio->status = inio->status;
+
+		inio->buffer_id = 0;
+		inport->buffers[0].outbuf->datas[0].chunk->size = 0;
+	}
 
 	if (SPA_UNLIKELY(inio->buffer_id >= inport->n_buffers))
 		return inio->status = -EINVAL;
@@ -783,6 +798,12 @@ static int impl_node_process(void *object)
 		flush_out = true;
 		break;
 	}
+	if (size == 0) {
+		size = sb->datas[0].maxsize;
+		memset(sb->datas[0].data, 0, size);
+		inport->offset = 0;
+		flush_in = draining = true;
+	}
 
 	if (this->io_rate_match) {
 		if (SPA_FLAG_IS_SET(this->io_rate_match->flags, SPA_IO_RATE_MATCH_FLAG_ACTIVE)) {
@@ -826,18 +847,19 @@ static int impl_node_process(void *object)
 	if (inport->offset >= size || flush_in) {
 		inio->status = SPA_STATUS_NEED_DATA;
 		inport->offset = 0;
-		SPA_FLAG_SET(res, SPA_STATUS_NEED_DATA);
-		spa_log_trace_fp(this->log, NAME " %p: return input buffer", this);
+		SPA_FLAG_SET(res, inio->status);
+		spa_log_trace_fp(this->log, NAME " %p: return input buffer of %zd samples", this, size / sizeof(float));
 	}
 
 	outport->offset += out_len * sizeof(float);
 	if (outport->offset > 0 && (outport->offset >= maxsize || flush_out)) {
 		outio->status = SPA_STATUS_HAVE_DATA;
 		outio->buffer_id = dbuf->id;
+		spa_log_trace_fp(this->log, NAME " %p: have output buffer of %zd samples", this, outport->offset / sizeof(float));
 		dequeue_buffer(this, dbuf);
 		outport->offset = 0;
+		this->drained = draining;
 		SPA_FLAG_SET(res, SPA_STATUS_HAVE_DATA);
-		spa_log_trace_fp(this->log, NAME " %p: have output buffer", this);
 	}
 	if (out_len == 0 && this->peaks) {
 		outio->status = SPA_STATUS_HAVE_DATA;
