@@ -329,6 +329,8 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	case SPA_NODE_COMMAND_Start:
 		this->started = true;
 		break;
+	case SPA_NODE_COMMAND_Suspend:
+		/* fallthrough */
 	case SPA_NODE_COMMAND_Pause:
 		this->started = false;
 		break;
@@ -409,7 +411,7 @@ static int port_enum_formats(void *object,
 				SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_audio),
 				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
 				SPA_FORMAT_AUDIO_format,   SPA_POD_CHOICE_ENUM_Id(18,
-							SPA_AUDIO_FORMAT_F32,
+							SPA_AUDIO_FORMAT_F32P,
 							SPA_AUDIO_FORMAT_F32P,
 							SPA_AUDIO_FORMAT_F32,
 							SPA_AUDIO_FORMAT_F32_OE,
@@ -419,12 +421,12 @@ static int port_enum_formats(void *object,
 							SPA_AUDIO_FORMAT_S24_32P,
 							SPA_AUDIO_FORMAT_S24_32,
 							SPA_AUDIO_FORMAT_S24_32_OE,
-							SPA_AUDIO_FORMAT_S24P,
-							SPA_AUDIO_FORMAT_S24,
-							SPA_AUDIO_FORMAT_S24_OE,
 							SPA_AUDIO_FORMAT_S16P,
 							SPA_AUDIO_FORMAT_S16,
 							SPA_AUDIO_FORMAT_S16_OE,
+							SPA_AUDIO_FORMAT_S24P,
+							SPA_AUDIO_FORMAT_S24,
+							SPA_AUDIO_FORMAT_S24_OE,
 							SPA_AUDIO_FORMAT_U8P,
 							SPA_AUDIO_FORMAT_U8),
 				SPA_FORMAT_AUDIO_rate,     SPA_POD_CHOICE_RANGE_Int(
