@@ -64,7 +64,8 @@ struct props {
 
 struct buffer {
 	uint32_t id;
-	unsigned int outstanding:1;
+#define BUFFER_FLAG_OUT	(1<<0)
+	uint32_t flags;
 	struct spa_buffer *buf;
 	struct spa_meta_header *h;
 	struct spa_list link;
@@ -116,7 +117,6 @@ struct impl {
 
 	struct spa_source source;
 	int timerfd;
-	int threshold;
 	struct spa_source flush_source;
 
 	struct spa_io_clock *clock;
@@ -559,6 +559,7 @@ static int flush_data(struct impl *this, uint64_t now_time)
 	struct port *port = &this->port;
 
 	total_frames = 0;
+again:
 	while (!spa_list_is_empty(&port->ready)) {
 		uint8_t *src;
 		uint32_t n_bytes, n_frames;
@@ -592,7 +593,8 @@ static int flush_data(struct impl *this, uint64_t now_time)
 
 			if (written < 0 && written != -ENOSPC) {
 				spa_list_remove(&b->link);
-				b->outstanding = true;
+				SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
+				this->port.io->buffer_id = b->id;
 				spa_log_trace(this->log, NAME " %p: error %s, reuse buffer %u",
 						this, spa_strerror(written), b->id);
 				spa_node_call_reuse_buffer(&this->callbacks, 0, b->id);
@@ -607,8 +609,9 @@ static int flush_data(struct impl *this, uint64_t now_time)
 
 		if (port->ready_offset >= d[0].chunk->size) {
 			spa_list_remove(&b->link);
-			b->outstanding = true;
+			SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 			spa_log_trace(this->log, NAME " %p: reuse buffer %u", this, b->id);
+			this->port.io->buffer_id = b->id;
 
 			spa_node_call_reuse_buffer(&this->callbacks, 0, b->id);
 			port->ready_offset = 0;
@@ -639,6 +642,8 @@ static int flush_data(struct impl *this, uint64_t now_time)
 			increase_bitpool(this);
 			this->last_error = now_time;
 		}
+		if (!spa_list_is_empty(&port->ready))
+			goto again;
 	}
 
 	this->flush_source.mask = 0;
@@ -678,6 +683,14 @@ static int flush_data(struct impl *this, uint64_t now_time)
 		spa_system_timerfd_settime(this->data_system, this->timerfd, SPA_FD_TIMER_ABSTIME, &ts, NULL);
 		this->source.mask = SPA_IO_IN;
 		spa_loop_update_source(this->data_loop, &this->source);
+
+		if (this->clock) {
+			this->clock->nsec = now_time;
+			this->clock->position = this->sample_count;
+			this->clock->delay = queued;
+			this->clock->rate_diff = 1.0f;
+			this->clock->next_nsec = SPA_TIMESPEC_TO_NSEC(&ts.it_value);
+		}
 	} else {
 		this->start_time = now_time;
 		this->sample_time = 0;
@@ -1212,7 +1225,6 @@ static int port_set_format(struct impl *this, struct port *port,
 		port->frame_size = info.info.raw.channels * 2;
 		port->current_format = info;
 		port->have_format = true;
-		this->threshold = this->props.min_latency;
 	}
 
 	port->info.change_mask |= SPA_PORT_CHANGE_MASK_PARAMS;
@@ -1283,7 +1295,7 @@ impl_node_port_use_buffers(void *object,
 
 		b->buf = buffers[i];
 		b->id = i;
-		b->outstanding = true;
+		SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
@@ -1291,7 +1303,6 @@ impl_node_port_use_buffers(void *object,
 			spa_log_error(this->log, NAME " %p: need mapped memory", this);
 			return -EINVAL;
 		}
-		this->threshold = buffers[i]->datas[0].maxsize / port->frame_size;
 	}
 	port->n_buffers = n_buffers;
 
@@ -1350,7 +1361,7 @@ static int impl_node_process(void *object)
 	if (io->status == SPA_STATUS_HAVE_DATA && io->buffer_id < port->n_buffers) {
 		struct buffer *b = &port->buffers[io->buffer_id];
 
-		if (!b->outstanding) {
+		if (!SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
 			spa_log_warn(this->log, NAME " %p: buffer %u in use", this, io->buffer_id);
 			io->status = -EINVAL;
 			return -EINVAL;
@@ -1359,12 +1370,9 @@ static int impl_node_process(void *object)
 		spa_log_trace(this->log, NAME " %p: queue buffer %u", this, io->buffer_id);
 
 		spa_list_append(&port->ready, &b->link);
-		b->outstanding = false;
+		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
 		port->need_data = false;
 
-		this->threshold = SPA_MIN(b->buf->datas[0].chunk->size / port->frame_size,
-				this->props.max_latency);
-
 		flush_data(this, now_time);
 
 		io->status = SPA_STATUS_OK;
