@@ -75,6 +75,7 @@ struct spa_bt_monitor {
 
 	struct spa_bt_backend *backend_hsp_native;
 	struct spa_bt_backend *backend_ofono;
+	struct spa_bt_backend *backend_hsphfpd;
 };
 
 static inline void add_dict(struct spa_pod_builder *builder, const char *key, const char *val)
@@ -816,6 +817,19 @@ struct spa_bt_transport *spa_bt_transport_find(struct spa_bt_monitor *monitor, c
 	return NULL;
 }
 
+struct spa_bt_transport *spa_bt_transport_find_full(struct spa_bt_monitor *monitor,
+                                                    bool (*callback) (struct spa_bt_transport *t, const void *data),
+                                                    const void *data)
+{
+	struct spa_bt_transport *t;
+
+	spa_list_for_each(t, &monitor->transport_list, link)
+		if (callback(t, data) == true)
+			return t;
+	return NULL;
+}
+
+
 struct spa_bt_transport *spa_bt_transport_create(struct spa_bt_monitor *monitor, char *path, size_t extra)
 {
 	struct spa_bt_transport *t;
@@ -1634,6 +1648,9 @@ impl_device_add_listener(void *object, struct spa_hook *listener,
 	if (this->backend_ofono)
 		backend_ofono_add_filters(this->backend_ofono);
 
+	if (this->backend_hsphfpd)
+		backend_hsphfpd_add_filters(this->backend_hsphfpd);
+
         spa_hook_list_join(&this->hooks, &save);
 
 	return 0;
@@ -1687,6 +1704,11 @@ static int impl_clear(struct spa_handle *handle)
 		monitor->backend_ofono = NULL;
 	}
 
+	if (monitor->backend_hsphfpd) {
+		backend_hsphfpd_free(monitor->backend_hsphfpd);
+		monitor->backend_hsphfpd = NULL;
+	}
+
 	return 0;
 }
 
@@ -1744,6 +1766,7 @@ impl_init(const struct spa_handle_factory *factory,
 
 	this->backend_hsp_native = backend_hsp_native_new(this, this->conn, support, n_support);
 	this->backend_ofono = backend_ofono_new(this, this->conn, support, n_support);
+	this->backend_hsphfpd = backend_hsphfpd_new(this, this->conn, support, n_support);
 
 	return 0;
 }
