@@ -38,8 +38,10 @@ int pw_data_loop_wait(struct pw_data_loop *this, int timeout)
 	int res;
 
 	while (true) {
-		if (!this->running)
-			return -ECANCELED;
+		if (!this->running) {
+			res = -ECANCELED;
+			break;
+		}
 		if ((res = pw_loop_iterate(this->loop, timeout)) < 0) {
 			if (errno == EINTR)
 				continue;
@@ -252,3 +254,16 @@ bool pw_data_loop_in_thread(struct pw_data_loop * loop)
 {
 	return pthread_equal(loop->thread, pthread_self());
 }
+
+SPA_EXPORT
+int pw_data_loop_invoke(struct pw_data_loop *loop,
+		spa_invoke_func_t func, uint32_t seq, const void *data, size_t size,
+		bool block, void *user_data)
+{
+	int res;
+	if (loop->running)
+		res = pw_loop_invoke(loop->loop, func, seq, data, size, block, user_data);
+	else
+		res = func(loop->loop->loop, false, seq, data, size, user_data);
+	return res;
+}
