@@ -46,6 +46,7 @@ struct impl {
 	struct pw_impl_node this;
 
 	enum pw_node_state pending;
+	uint32_t pending_id;
 
 	struct pw_work_queue *work;
 
@@ -201,16 +202,17 @@ static int start_node(struct pw_impl_node *this)
 
 static void emit_info_changed(struct pw_impl_node *node)
 {
-	struct pw_resource *resource;
 
 	if (node->info.change_mask == 0)
 		return;
 
 	pw_impl_node_emit_info_changed(node, &node->info);
 
-	if (node->global)
+	if (node->global) {
+		struct pw_resource *resource;
 		spa_list_for_each(resource, &node->global->resource_list, link)
 			pw_node_resource_info(resource, &node->info);
+	}
 
 	node->info.change_mask = 0;
 }
@@ -286,7 +288,7 @@ do_node_add(struct spa_loop *loop,
 	return 0;
 }
 
-static void node_update_state(struct pw_impl_node *node, enum pw_node_state state, char *error)
+static void node_update_state(struct pw_impl_node *node, enum pw_node_state state, int res, char *error)
 {
 	struct impl *impl = SPA_CONTAINER_OF(node, struct impl, this);
 	enum pw_node_state old = node->info.state;
@@ -322,6 +324,12 @@ static void node_update_state(struct pw_impl_node *node, enum pw_node_state stat
 
 	node->info.change_mask |= PW_NODE_CHANGE_MASK_STATE;
 	emit_info_changed(node);
+
+	if (state == PW_NODE_STATE_ERROR && node->global) {
+		struct pw_resource *resource;
+		spa_list_for_each(resource, &node->global->resource_list, link)
+			pw_resource_error(resource, res, error);
+	}
 }
 
 static int suspend_node(struct pw_impl_node *this)
@@ -361,7 +369,7 @@ static int suspend_node(struct pw_impl_node *this)
 	if (res < 0 && res != -EIO)
 		pw_log_warn(NAME" %p: suspend node error %s", this, spa_strerror(res));
 
-	node_update_state(this, PW_NODE_STATE_SUSPENDED, NULL);
+	node_update_state(this, PW_NODE_STATE_SUSPENDED, 0, NULL);
 
 	return res;
 }
@@ -685,7 +693,7 @@ int pw_impl_node_initialized(struct pw_impl_node *this)
 {
 	pw_log_debug(NAME" %p initialized", this);
 	pw_impl_node_emit_initialized(this);
-	node_update_state(this, PW_NODE_STATE_SUSPENDED, NULL);
+	node_update_state(this, PW_NODE_STATE_SUSPENDED, 0, NULL);
 	return 0;
 }
 
@@ -837,8 +845,9 @@ static void check_properties(struct pw_impl_node *node)
 			if (quantum_size != node->quantum_size) {
 				pw_log_debug(NAME" %p: latency '%s' quantum %u/%u",
 						node, str, quantum_size, context->defaults.clock_rate);
-				pw_log_info("(%s-%u) quantum %u/%u", node->name, node->info.id,
-						quantum_size, context->defaults.clock_rate);
+				pw_log_info("(%s-%u) latency:%s ->quantum %u/%u", node->name,
+						node->info.id, str, quantum_size,
+						context->defaults.clock_rate);
 				node->quantum_size = quantum_size;
 				do_recalc = true;
 			}
@@ -875,7 +884,12 @@ static void dump_states(struct pw_impl_node *driver)
 		struct pw_node_activation_state *state = &a->state[0];
 		if (t->node == NULL)
 			continue;
-		pw_log_warn("(%s-%u) state:%p pending:%d/%d s:%"PRIu64" a:%"PRIu64" f:%"PRIu64
+		if (a->status == PW_NODE_ACTIVATION_TRIGGERED ||
+		    a->status == PW_NODE_ACTIVATION_AWAKE) {
+			pw_log_warn("(%s-%u) client too slow! status:%s",
+				t->node->name, t->node->info.id, str_status(a->status));
+		}
+		pw_log_debug("(%s-%u) state:%p pending:%d/%d s:%"PRIu64" a:%"PRIu64" f:%"PRIu64
 				" waiting:%"PRIu64" process:%"PRIu64" status:%s sync:%d",
 				t->node->name, t->node->info.id, state,
 				state->pending, state->required,
@@ -1007,7 +1021,8 @@ static void node_on_fd_events(struct spa_source *source)
 		if (SPA_UNLIKELY(spa_system_eventfd_read(data_system, this->source.fd, &cmd) < 0))
 			pw_log_warn(NAME" %p: read failed %m", this);
 		else if (SPA_UNLIKELY(cmd > 1))
-			pw_log_warn(NAME" %p: missed %"PRIu64" wakeups", this, cmd - 1);
+			pw_log_warn("(%s-%u) client missed %"PRIu64" wakeups",
+				this->name, this->info.id, cmd - 1);
 
 		pw_log_trace_fp(NAME" %p: got process", this);
 		this->rt.target.signal(this->rt.target.data);
@@ -1100,6 +1115,7 @@ struct pw_impl_node *pw_context_create_node(struct pw_context *context,
 		res = -errno;
 		goto error_clean;
 	}
+	impl->pending_id = SPA_ID_INVALID;
 
 	this->data_loop = context->data_loop;
 
@@ -1320,7 +1336,8 @@ static void node_event(void *data, const struct spa_event *event)
 	switch (SPA_NODE_EVENT_ID(event)) {
 	case SPA_NODE_EVENT_Error:
 		impl->last_error = -EFAULT;
-		node_update_state(node, PW_NODE_STATE_ERROR, strdup("error"));
+		node_update_state(node, PW_NODE_STATE_ERROR,
+				-EFAULT, strdup("Received error event"));
 		break;
 	default:
 		pw_log_debug("unhandled event");
@@ -1445,7 +1462,7 @@ static int node_ready(void *data, int status)
 		if (SPA_UNLIKELY(state->pending > 0)) {
 			pw_context_driver_emit_incomplete(node->context, node);
 			if (ratelimit_test(&node->rt.rate_limit, a->signal_time)) {
-				pw_log_warn("(%s-%u) graph not finished: state:%p quantum:%"PRIu64
+				pw_log_debug("(%s-%u) graph not finished: state:%p quantum:%"PRIu64
 						" pending %d/%d", node->name, node->info.id,
 						state, a->position.clock.duration,
 						state->pending, state->required);
@@ -1852,6 +1869,8 @@ static void on_state_complete(void *obj, void *data, int res, uint32_t seq)
 	enum pw_node_state state = SPA_PTR_TO_INT(data);
 	char *error = NULL;
 
+	impl->pending_id = SPA_ID_INVALID;
+
 	pw_log_debug(NAME" %p: state complete res:%d seq:%d", node, res, seq);
 	if (impl->last_error < 0) {
 		res = impl->last_error;
@@ -1860,12 +1879,13 @@ static void on_state_complete(void *obj, void *data, int res, uint32_t seq)
 	if (SPA_RESULT_IS_ERROR(res)) {
 		if (node->info.state == PW_NODE_STATE_SUSPENDED) {
 			state = PW_NODE_STATE_SUSPENDED;
+			res = 0;
 		} else {
 			error = spa_aprintf("error changing node state: %s", spa_strerror(res));
 			state = PW_NODE_STATE_ERROR;
 		}
 	}
-	node_update_state(node, state, error);
+	node_update_state(node, state, res, error);
 }
 
 static void node_activate(struct pw_impl_node *this)
@@ -1938,12 +1958,14 @@ int pw_impl_node_set_state(struct pw_impl_node *node, enum pw_node_state state)
 	if (SPA_RESULT_IS_ASYNC(res)) {
 		res = spa_node_sync(node->node, res);
 	}
-	impl->pending = state;
 
-	if (old != state)
-		pw_work_queue_add(impl->work,
+	if (old != state) {
+		impl->pending = state;
+		if (impl->pending_id != SPA_ID_INVALID)
+			pw_work_queue_cancel(impl->work, node, impl->pending_id);
+		impl->pending_id = pw_work_queue_add(impl->work,
 				node, res, on_state_complete, SPA_INT_TO_PTR(state));
-
+	}
 	return res;
 }
 
