@@ -114,15 +114,14 @@ static int process_clock(struct data *d, const struct spa_pod *pod, struct point
 
 static int process_driver_block(struct data *d, const struct spa_pod *pod, struct point *point)
 {
-	union {
-		char *s;
-	} dummy;
-	uint32_t driver_id;
+	char *name = NULL;
+	uint32_t driver_id = 0;
 	struct measurement driver;
 
+	spa_zero(driver);
 	spa_pod_parse_struct(pod,
 			SPA_POD_Int(&driver_id),
-			SPA_POD_String(&dummy.s),
+			SPA_POD_String(&name),
 			SPA_POD_Long(&driver.prev_signal),
 			SPA_POD_Long(&driver.signal),
 			SPA_POD_Long(&driver.awake),
@@ -169,8 +168,8 @@ static int add_follower(struct data *d, uint32_t id, const char *name)
 
 static int process_follower_block(struct data *d, const struct spa_pod *pod, struct point *point)
 {
-	uint32_t id;
-	const char *name;
+	uint32_t id = 0;
+	const char *name =  NULL;
 	struct measurement m;
 	int idx;
 
@@ -541,12 +540,12 @@ static void do_quit(void *data, int signal_number)
 static void show_help(const char *name)
 {
         fprintf(stdout, "%s [options]\n"
-             "  -h, --help                            Show this help\n"
-             "  -v, --version                         Show version\n"
-             "  -r, --remote                          Remote daemon name\n"
-             "  -o, --output                          Profiler output name (default \"%s\")\n",
-	     name,
-	     DEFAULT_FILENAME);
+		"  -h, --help                            Show this help\n"
+		"      --version                         Show version\n"
+		"  -r, --remote                          Remote daemon name\n"
+		"  -o, --output                          Profiler output name (default \"%s\")\n",
+		name,
+		DEFAULT_FILENAME);
 }
 
 int main(int argc, char *argv[])
@@ -556,22 +555,22 @@ int main(int argc, char *argv[])
 	const char *opt_remote = NULL;
 	const char *opt_output = DEFAULT_FILENAME;
 	static const struct option long_options[] = {
-		{"help",	0, NULL, 'h'},
-		{"version",	0, NULL, 'v'},
-		{"remote",	1, NULL, 'r'},
-		{"output",	1, NULL, 'o'},
-		{NULL,		0, NULL, 0}
+		{ "help",	no_argument,		NULL, 'h' },
+		{ "version",	no_argument,		NULL, 'V' },
+		{ "remote",	required_argument,	NULL, 'r' },
+		{ "output",	required_argument,	NULL, 'o' },
+		{ NULL, 0, NULL, 0}
 	};
 	int c;
 
 	pw_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "hvr:o:", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hVr:o:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
 			show_help(argv[0]);
 			return 0;
-		case 'v':
+		case 'V':
 			fprintf(stdout, "%s\n"
 				"Compiled with libpipewire %s\n"
 				"Linked with libpipewire %s\n",
@@ -586,13 +585,14 @@ int main(int argc, char *argv[])
 			opt_remote = optarg;
 			break;
 		default:
+			show_help(argv[0]);
 			return -1;
 		}
 	}
 
 	data.loop = pw_main_loop_new(NULL);
 	if (data.loop == NULL) {
-		fprintf(stderr, "Can't create data loop: %m");
+		fprintf(stderr, "Can't create data loop: %m\n");
 		return -1;
 	}
 
@@ -602,7 +602,7 @@ int main(int argc, char *argv[])
 
 	data.context = pw_context_new(l, NULL, 0);
 	if (data.context == NULL) {
-		fprintf(stderr, "Can't create context: %m");
+		fprintf(stderr, "Can't create context: %m\n");
 		return -1;
 	}
 
@@ -614,7 +614,7 @@ int main(int argc, char *argv[])
 				NULL),
 			0);
 	if (data.core == NULL) {
-		fprintf(stderr, "Can't connect: %m");
+		fprintf(stderr, "Can't connect: %m\n");
 		return -1;
 	}
 
@@ -622,7 +622,7 @@ int main(int argc, char *argv[])
 
 	data.output = fopen(data.filename, "w");
 	if (data.output == NULL) {
-		fprintf(stderr, "Can't open file %s: %m", data.filename);
+		fprintf(stderr, "Can't open file %s: %m\n", data.filename);
 		return -1;
 	}
 
@@ -641,6 +641,8 @@ int main(int argc, char *argv[])
 
 	pw_main_loop_run(data.loop);
 
+	pw_proxy_destroy((struct pw_proxy*)data.profiler);
+	pw_proxy_destroy((struct pw_proxy*)data.registry);
 	pw_context_destroy(data.context);
 	pw_main_loop_destroy(data.loop);
 
@@ -648,5 +650,7 @@ int main(int argc, char *argv[])
 
 	dump_scripts(&data);
 
+	pw_deinit();
+
 	return 0;
 }
